@@ -1,4 +1,4 @@
-# Copyright 2009-2010 10gen, Inc.
+# Copyright 2009-2012 10gen, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -51,6 +51,7 @@ class SON(dict):
     unicode                              code           bson -> py
     `bson.code.Code`                     code           py -> bson
     unicode                              symbol         bson -> py
+    bytes (Python 3) [#bytes]_           binary         both
     ===================================  =============  ===================
 
     Note that to save binary data it must be wrapped as an instance of
@@ -58,12 +59,17 @@ class SON(dict):
     and retrieved as unicode.
 
     .. [#int] A Python int will be saved as a BSON int32 or BSON int64 depending
-       on its size. A BSON int32 will always decode to a Python int. A BSON int64
-       will always decode to a Python long.
+       on its size. A BSON int32 will always decode to a Python int. In Python 2.x
+       a BSON int64 will always decode to a Python long. In Python 3.x a BSON
+       int64 will decode to a Python int since there is no longer a long type.
     .. [#dt] datetime.datetime instances will be rounded to the nearest
        millisecond when saved
     .. [#dt2] all datetime.datetime instances are treated as *naive*. clients
        should always use UTC.
+    .. [#bytes] The bytes type from Python 3.x is encoded as BSON binary with
+       subtype 0. In Python 3.x it will be decoded back to bytes. In Python 2.x
+       it will be decoded to an instance of :class:`~bson.binary.Binary` with
+       subtype 0.
     """
 
     def __init__(self, data=None, **kwargs):
@@ -72,6 +78,11 @@ class SON(dict):
         self.update(data)
         self.update(kwargs)
 
+    def __new__(cls, *args, **kwargs):
+        instance = super(SON, cls).__new__(cls, *args, **kwargs)
+        instance.__keys = []
+        return instance
+
     def __repr__(self):
         result = []
         for key in self.__keys:
@@ -125,7 +136,7 @@ class SON(dict):
         return [v for _, v in self.iteritems()]
 
     def items(self):
-        return list(self.iteritems())
+        return [(key, self[key]) for key in self]
 
     def clear(self):
         for key in self.keys():
@@ -181,11 +192,11 @@ class SON(dict):
         except KeyError:
             return default
 
-    def __cmp__(self, other):
+    def __eq__(self, other):
         if isinstance(other, SON):
-            return cmp((dict(self.iteritems()), self.keys()),
-                       (dict(other.iteritems()), other.keys()))
-        return cmp(dict(self.iteritems()), other)
+            return (len(self) == len(other) and
+                    dict(self.items()) == dict(other.items()))
+        return dict(self.items()) == other
 
     def __len__(self):
         return len(self.keys())
