@@ -1,4 +1,4 @@
-# Copyright 2009-2010 10gen, Inc.
+# Copyright 2009-2012 10gen, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -80,22 +80,22 @@ class GridFS(object):
     def put(self, data, **kwargs):
         """Put data in GridFS as a new file.
 
-        Equivalent to doing:
+        Equivalent to doing::
 
-        >>> f = new_file(**kwargs)
-        >>> try:
-        >>>     f.write(data)
-        >>> finally:
-        >>>     f.close()
+          try:
+              f = new_file(**kwargs)
+              f.write(data)
+          finally
+              f.close()
 
-        `data` can be either an instance of :class:`str` or a
-        file-like object providing a :meth:`read` method. If an
-        `encoding` keyword argument is passed, `data` can also be a
-        :class:`unicode` instance, which will be encoded as `encoding`
-        before being written. Any keyword arguments will be passed
-        through to the created file - see
-        :meth:`~gridfs.grid_file.GridIn` for possible
-        arguments. Returns the ``"_id"`` of the created file.
+        `data` can be either an instance of :class:`str` (:class:`bytes`
+        in python 3) or a file-like object providing a :meth:`read` method.
+        If an `encoding` keyword argument is passed, `data` can also be a
+        :class:`unicode` (:class:`str` in python 3) instance, which will
+        be encoded as `encoding` before being written. Any keyword arguments
+        will be passed through to the created file - see
+        :meth:`~gridfs.grid_file.GridIn` for possible arguments. Returns the
+        ``"_id"`` of the created file.
 
         If the ``"_id"`` of the file is manually specified, it must
         not already exist in GridFS. Otherwise
