@@ -1,4 +1,4 @@
-# Copyright 2009-2010 10gen, Inc.
+# Copyright 2009-2012 10gen, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -25,8 +25,6 @@ from pymongo import (common,
 from pymongo.cursor import Cursor
 from pymongo.errors import ConfigurationError, InvalidName, InvalidOperation
 
-_ZERO = "\x00\x00\x00\x00"
-
 
 def _gen_index_name(keys):
     """Generate an index name from the set of fields it is over.
@@ -38,11 +36,11 @@ class Collection(common.BaseObject):
     """A Mongo collection.
     """
 
-    def __init__(self, database, name, options=None, create=False, **kwargs):
+    def __init__(self, database, name, create=False, **kwargs):
         """Get / create a Mongo collection.
 
         Raises :class:`TypeError` if `name` is not an instance of
-        :class:`basestring`. Raises
+        :class:`basestring` (:class:`str` in python 3). Raises
         :class:`~pymongo.errors.InvalidName` if `name` is not a valid
         collection name. Any additional keyword arguments will be used
         as options passed to the create command. See
@@ -57,16 +55,20 @@ class Collection(common.BaseObject):
         :Parameters:
           - `database`: the database to get a collection from
           - `name`: the name of the collection to get
-          - `options`: DEPRECATED dictionary of collection options
           - `create` (optional): if ``True``, force collection
             creation even without options being set
           - `**kwargs` (optional): additional keyword arguments will
             be passed as options for the create collection command
 
+        .. versionchanged:: 2.2
+           Removed deprecated argument: options
+
         .. versionadded:: 2.1
            uuid_subtype attribute
+
         .. versionchanged:: 1.5
            deprecating `options` in favor of kwargs
+
         .. versionadded:: 1.5
            the `create` parameter
 
@@ -79,17 +81,8 @@ class Collection(common.BaseObject):
                              **(database.get_lasterror_options()))
 
         if not isinstance(name, basestring):
-            raise TypeError("name must be an instance of basestring")
-
-        if options is not None:
-            warnings.warn("the options argument to Collection is deprecated "
-                          "and will be removed. please use kwargs instead.",
-                          DeprecationWarning)
-            if not isinstance(options, dict):
-                raise TypeError("options must be an instance of dict")
-            options.update(kwargs)
-        elif kwargs:
-            options = kwargs
+            raise TypeError("name must be an instance "
+                            "of %s" % (basestring.__name__,))
 
         if not name or ".." in name:
             raise InvalidName("collection names cannot be empty")
@@ -108,8 +101,8 @@ class Collection(common.BaseObject):
         self.__name = unicode(name)
         self.__uuid_subtype = OLD_UUID_SUBTYPE
         self.__full_name = u"%s.%s" % (self.__database.name, self.__name)
-        if create or options is not None:
-            self.__create(options)
+        if create or kwargs:
+            self.__create(kwargs)
 
     def __create(self, options):
         """Sends a create command with the given options.
@@ -140,10 +133,11 @@ class Collection(common.BaseObject):
     def __repr__(self):
         return "Collection(%r, %r)" % (self.__database, self.__name)
 
-    def __cmp__(self, other):
+    def __eq__(self, other):
         if isinstance(other, Collection):
-            return cmp((self.__database, self.__name),
-                       (other.__database, other.__name))
+            us = (self.__database, self.__name)
+            them = (other.__database, other.__name)
+            return us == them
         return NotImplemented
 
     @property
@@ -631,15 +625,14 @@ class Collection(common.BaseObject):
         """
         return self.find().count()
 
-    def create_index(self, key_or_list, deprecated_unique=None,
-                     ttl=300, **kwargs):
+    def create_index(self, key_or_list, ttl=300, **kwargs):
         """Creates an index on this collection.
 
         Takes either a single key or a list of (key, direction) pairs.
-        The key(s) must be an instance of :class:`basestring`, and the
-        directions must be one of (:data:`~pymongo.ASCENDING`,
-        :data:`~pymongo.DESCENDING`, :data:`~pymongo.GEO2D`). Returns
-        the name of the created index.
+        The key(s) must be an instance of :class:`basestring`
+        (:class:`str` in python 3), and the directions must be one of
+        (:data:`~pymongo.ASCENDING`, :data:`~pymongo.DESCENDING`,
+        :data:`~pymongo.GEO2D`). Returns the name of the created index.
 
         To create a single key index on the key ``'mike'`` we just use
         a string argument:
@@ -669,7 +662,6 @@ class Collection(common.BaseObject):
         :Parameters:
           - `key_or_list`: a single key or a list of (key, direction)
             pairs specifying the index to create
-          - `deprecated_unique`: DEPRECATED - use `unique` as a kwarg
           - `ttl` (optional): time window (in seconds) during which
             this index will be recognized by subsequent calls to
             :meth:`ensure_index` - see documentation for
@@ -678,6 +670,9 @@ class Collection(common.BaseObject):
             options (see the above list) should be passed as keyword
             arguments
 
+        .. versionchanged:: 2.2
+           Removed deprecated argument: deprecated_unique
+
         .. versionchanged:: 1.5.1
            Accept kwargs to support all index creation options.
 
@@ -693,12 +688,6 @@ class Collection(common.BaseObject):
 
         index = {"key": index_doc, "ns": self.__full_name}
 
-        if deprecated_unique is not None:
-            warnings.warn("using a positional arg to specify unique is "
-                          "deprecated, please use kwargs",
-                          DeprecationWarning)
-            index["unique"] = deprecated_unique
-
         name = "name" in kwargs and kwargs["name"] or _gen_index_name(keys)
         index["name"] = name
 
@@ -719,15 +708,15 @@ class Collection(common.BaseObject):
 
         return name
 
-    def ensure_index(self, key_or_list, deprecated_unique=None,
-                     ttl=300, **kwargs):
+    def ensure_index(self, key_or_list, ttl=300, **kwargs):
         """Ensures that an index exists on this collection.
 
         Takes either a single key or a list of (key, direction) pairs.
-        The key(s) must be an instance of :class:`basestring`, and the
-        direction(s) must be one of (:data:`~pymongo.ASCENDING`,
-        :data:`~pymongo.DESCENDING`, :data:`~pymongo.GEO2D`). See
-        :meth:`create_index` for a detailed example.
+        The key(s) must be an instance of :class:`basestring`
+        (:class:`str` in python 3), and the direction(s) must be one of
+        (:data:`~pymongo.ASCENDING`, :data:`~pymongo.DESCENDING`,
+        :data:`~pymongo.GEO2D`). See :meth:`create_index` for a detailed
+        example.
 
         Unlike :meth:`create_index`, which attempts to create an index
         unconditionally, :meth:`ensure_index` takes advantage of some
@@ -765,7 +754,6 @@ class Collection(common.BaseObject):
         :Parameters:
           - `key_or_list`: a single key or a list of (key, direction)
             pairs specifying the index to create
-          - `deprecated_unique`: DEPRECATED - use `unique` as a kwarg
           - `ttl` (optional): time window (in seconds) during which
             this index will be recognized by subsequent calls to
             :meth:`ensure_index`
@@ -773,6 +761,9 @@ class Collection(common.BaseObject):
             options (see the above list) should be passed as keyword
             arguments
 
+        .. versionchanged:: 2.2
+           Removed deprecated argument: deprecated_unique
+
         .. versionchanged:: 1.5.1
            Accept kwargs to support all index creation options.
 
@@ -789,8 +780,7 @@ class Collection(common.BaseObject):
 
         if not self.__database.connection._cached(self.__database.name,
                                                   self.__name, name):
-            return self.create_index(key_or_list, deprecated_unique,
-                                     ttl, **kwargs)
+            return self.create_index(key_or_list, ttl, **kwargs)
         return None
 
     def drop_indexes(self):
@@ -813,7 +803,9 @@ class Collection(common.BaseObject):
         specifier should be a list of (key, direction) pairs. Raises
         TypeError if index is not an instance of (str, unicode, list).
 
-        .. warning:: if a custom name was used on index creation (by
+        .. warning::
+
+          if a custom name was used on index creation (by
           passing the `name` parameter to :meth:`create_index` or
           :meth:`ensure_index`) the index **must** be dropped by name.
 
@@ -899,8 +891,7 @@ class Collection(common.BaseObject):
 
     # TODO key and condition ought to be optional, but deprecation
     # could be painful as argument order would have to change.
-    def group(self, key, condition, initial, reduce, finalize=None,
-              command=True):
+    def group(self, key, condition, initial, reduce, finalize=None):
         """Perform a query similar to an SQL *group by* operation.
 
         Returns an array of grouped items.
@@ -908,10 +899,12 @@ class Collection(common.BaseObject):
         The `key` parameter can be:
 
           - ``None`` to use the entire document as a key.
-          - A :class:`list` of keys (each a :class:`basestring`) to group by.
-          - A :class:`basestring` or :class:`~bson.code.Code` instance
-            containing a JavaScript function to be applied to each
-            document, returning the key to group by.
+          - A :class:`list` of keys (each a :class:`basestring`
+            (:class:`str` in python 3)) to group by.
+          - A :class:`basestring` (:class:`str` in python 3), or
+            :class:`~bson.code.Code` instance containing a JavaScript
+            function to be applied to each document, returning the key
+            to group by.
 
         With :class:`~pymongo.replica_set_connection.ReplicaSetConnection`
         or :class:`~pymongo.master_slave_connection.MasterSlaveConnection`,
@@ -927,20 +920,17 @@ class Collection(common.BaseObject):
           - `initial`: initial value of the aggregation counter object
           - `reduce`: aggregation function as a JavaScript string
           - `finalize`: function to be called on each object in output list.
-          - `command` (optional): DEPRECATED if ``True``, run the group as a
-            command instead of in an eval - this option is deprecated and
-            will be removed in favor of running all groups as commands
+
+        .. versionchanged:: 2.2
+           Removed deprecated argument: command
 
         .. versionchanged:: 1.4
            The `key` argument can now be ``None`` or a JavaScript function,
            in addition to a :class:`list` of keys.
+
         .. versionchanged:: 1.3
            The `command` argument now defaults to ``True`` and is deprecated.
         """
-        if not command:
-            warnings.warn("eval-based groups are deprecated, and the "
-                          "command option will be removed.",
-                          DeprecationWarning)
 
         group = {}
         if isinstance(key, basestring):
@@ -967,9 +957,9 @@ class Collection(common.BaseObject):
 
         If operating in auth mode, client must be authorized as an
         admin to perform this operation. Raises :class:`TypeError` if
-        `new_name` is not an instance of :class:`basestring`. Raises
-        :class:`~pymongo.errors.InvalidName` if `new_name` is not a
-        valid collection name.
+        `new_name` is not an instance of :class:`basestring`
+        (:class:`str` in python 3). Raises :class:`~pymongo.errors.InvalidName`
+        if `new_name` is not a valid collection name.
 
         :Parameters:
           - `new_name`: new name for this collection
@@ -981,7 +971,8 @@ class Collection(common.BaseObject):
            support for accepting keyword arguments for rename options
         """
         if not isinstance(new_name, basestring):
-            raise TypeError("new_name must be an instance of basestring")
+            raise TypeError("new_name must be an instance "
+                            "of %s" % (basestring.__name__,))
 
         if not new_name or ".." in new_name:
             raise InvalidName("collection names cannot be empty")
@@ -1000,7 +991,7 @@ class Collection(common.BaseObject):
         in this collection.
 
         Raises :class:`TypeError` if `key` is not an instance of
-        :class:`basestring`.
+        :class:`basestring` (:class:`str` in python 3).
 
         To get the distinct values for a key in the result set of a
         query use :meth:`~pymongo.cursor.Cursor.distinct`.
@@ -1014,8 +1005,7 @@ class Collection(common.BaseObject):
         """
         return self.find().distinct(key)
 
-    def map_reduce(self, map, reduce, out, merge_output=False,
-                   reduce_output=False, full_response=False, **kwargs):
+    def map_reduce(self, map, reduce, out, full_response=False, **kwargs):
         """Perform a map/reduce operation on this collection.
 
         If `full_response` is ``False`` (default) returns a
@@ -1031,16 +1021,6 @@ class Collection(common.BaseObject):
             Note: `out` options are order sensitive. :class:`~bson.son.SON`
             can be used to specify multiple options.
             e.g. SON([('replace', <collection name>), ('db', <database name>)])
-          - `merge_output` (optional) DEPRECATED: Merge output into `out`.
-            If the same key exists in both the result set and the existing
-            output collection, the new key will overwrite the existing key.
-            Ignored if `out` is not an instance of `basestring`.
-          - `reduce_output` (optional) DEPRECATED: If documents exist for
-            a given key in the result set and in the existing output
-            collection, then a reduce operation (using the specified reduce
-            function) will be performed on the two values and the result will
-            be written to the output collection.
-            Ignored if `out` is not an instance of `basestring`.
           - `full_response` (optional): if ``True``, return full response to
             this command - otherwise just return the result collection
           - `**kwargs` (optional): additional arguments to the
@@ -1053,6 +1033,9 @@ class Collection(common.BaseObject):
 
         .. seealso:: :doc:`/examples/map_reduce`
 
+        .. versionchanged:: 2.2
+           Removed deprecated arguments: merge_output and reduce_output
+
         .. versionchanged:: 1.11+
            DEPRECATED The merge_output and reduce_output parameters.
 
@@ -1062,31 +1045,14 @@ class Collection(common.BaseObject):
 
         .. mongodoc:: mapreduce
         """
-        if merge_output or reduce_output:
-            warnings.warn("merge_output and reduce_output are deprecated, "
-                          "please pass {<merge|reduce>: <collection name>} "
-                          "as the 'out' parameter instead.",
-                          DeprecationWarning)
-        if merge_output and reduce_output:
-            raise InvalidOperation("Can't do both merge "
-                                   "and re-reduce of output.")
-
-        if isinstance(out, basestring):
-            if merge_output:
-                out_conf = {"merge": out}
-            elif reduce_output:
-                out_conf = {"reduce": out}
-            else:
-                out_conf = out
-        elif isinstance(out, dict):
-            out_conf = out
-        else:
-            raise TypeError("'out' must be an instance of basestring or dict")
+        if not isinstance(out, (basestring, dict)):
+            raise TypeError("'out' must be an instance of "
+                            "%s or dict" % (basestring.__name__,))
 
         response = self.__database.command("mapreduce", self.__name,
                                            uuid_subtype=self.__uuid_subtype,
                                            map=map, reduce=reduce,
-                                           out=out_conf, **kwargs)
+                                           out=out, **kwargs)
 
         if full_response or not response.get('result'):
             return response
