@@ -1,4 +1,4 @@
-# Copyright 2009-2010 10gen, Inc.
+# Copyright 2009-2012 10gen, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,11 +20,13 @@ try:
 except:  # for Python < 2.5
     import md5
     _md5func = md5.new
+import random
 import struct
 
 import bson
-from bson.son import SON
 import pymongo
+
+from bson.son import SON
 from pymongo.errors import (AutoReconnect,
                             OperationFailure,
                             TimeoutError)
@@ -131,13 +133,15 @@ def _password_digest(username, password):
     """Get a password digest to use for authentication.
     """
     if not isinstance(password, basestring):
-        raise TypeError("password must be an instance of basestring")
+        raise TypeError("password must be an instance "
+                        "of %s" % (basestring.__name__,))
     if not isinstance(username, basestring):
-        raise TypeError("username must be an instance of basestring")
+        raise TypeError("username must be an instance "
+                        "of %s" % (basestring.__name__,))
 
     md5hash = _md5func()
-    md5hash.update("%s:mongo:%s" % (username.encode('utf-8'),
-                                    password.encode('utf-8')))
+    data = "%s:mongo:%s" % (username, password)
+    md5hash.update(data.encode('utf-8'))
     return unicode(md5hash.hexdigest())
 
 
@@ -146,7 +150,8 @@ def _auth_key(nonce, username, password):
     """
     digest = _password_digest(username, password)
     md5hash = _md5func()
-    md5hash.update("%s%s%s" % (nonce, unicode(username), digest))
+    data = "%s%s%s" % (nonce, unicode(username), digest)
+    md5hash.update(data.encode('utf-8'))
     return unicode(md5hash.hexdigest())
 
 
@@ -162,7 +167,16 @@ def _fields_list_to_dict(fields):
     as_dict = {}
     for field in fields:
         if not isinstance(field, basestring):
-            raise TypeError("fields must be a list of key names as "
-                            "(string, unicode)")
+            raise TypeError("fields must be a list of key names, "
+                            "each an instance of %s" % (basestring.__name__,))
         as_dict[field] = 1
     return as_dict
+
+def shuffled(sequence):
+    """Returns a copy of the sequence (as a :class:`list`) which has been
+    shuffled by :func:`random.shuffle`.
+    """
+    out = list(sequence)
+    random.shuffle(out)
+    return out
+
