@@ -1,4 +1,4 @@
-# Copyright 2009-2010 10gen, Inc.
+# Copyright 2009-2012 10gen, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -26,6 +26,8 @@ import random
 import struct
 
 import bson
+from bson.binary import OLD_UUID_SUBTYPE
+from bson.py3compat import b
 from bson.son import SON
 try:
     from pymongo import _cmessage
@@ -35,7 +37,9 @@ except ImportError:
 from pymongo.errors import InvalidOperation
 
 
-__ZERO = "\x00\x00\x00\x00"
+__ZERO = b("\x00\x00\x00\x00")
+
+EMPTY  = b("")
 
 MAX_INT32 = 2147483647
 MIN_INT32 = -2147483648
@@ -76,7 +80,7 @@ def insert(collection_name, docs, check_keys,
     if not encoded:
         raise InvalidOperation("cannot do an empty bulk insert")
     max_bson_size = max(map(len, encoded))
-    data += "".join(encoded)
+    data += EMPTY.join(encoded)
     if safe:
         (_, insert_message) = __pack_message(2002, data)
         (request_id, error_message, _) = __last_error(last_error_args)
@@ -116,7 +120,8 @@ if _use_c:
 
 
 def query(options, collection_name, num_to_skip,
-          num_to_return, query, field_selector=None, uuid_subtype=4):
+          num_to_return, query, field_selector=None,
+          uuid_subtype=OLD_UUID_SUBTYPE):
     """Get a **query** message.
     """
     data = struct.pack("<I", options)
