@@ -1,4 +1,4 @@
-# Copyright 2009-2010 10gen, Inc.
+# Copyright 2009-2012 10gen, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,11 +16,11 @@
 
 import datetime
 import os
-import signal
 import sys
 import time
 import thread
 import unittest
+
 sys.path[0:0] = [""]
 
 from nose.plugins.skip import SkipTest
@@ -29,6 +29,7 @@ from bson.son import SON
 from bson.tz_util import utc
 from pymongo.connection import Connection
 from pymongo.database import Database
+from pymongo.pool import NO_REQUEST, NO_SOCKET_YET, SocketInfo
 from pymongo.errors import (AutoReconnect,
                             ConfigurationError,
                             ConnectionFailure,
@@ -36,11 +37,13 @@ from pymongo.errors import (AutoReconnect,
                             InvalidURI,
                             OperationFailure)
 from test import version
+from test.utils import server_is_master_with_slave, delay
+
+host = os.environ.get("DB_IP", "localhost")
+port = int(os.environ.get("DB_PORT", 27017))
 
 
 def get_connection(*args, **kwargs):
-    host = os.environ.get("DB_IP", "localhost")
-    port = int(os.environ.get("DB_PORT", 27017))
     return Connection(host, port, *args, **kwargs)
 
 
@@ -62,26 +65,26 @@ class TestConnection(unittest.TestCase):
     def test_constants(self):
         Connection.HOST = self.host
         Connection.PORT = self.port
-        self.assert_(Connection())
+        self.assertTrue(Connection())
 
         Connection.HOST = "somedomainthatdoesntexist.org"
         Connection.PORT = 123456789
         self.assertRaises(ConnectionFailure, Connection, connectTimeoutMS=600)
-        self.assert_(Connection(self.host, self.port))
+        self.assertTrue(Connection(self.host, self.port))
 
         Connection.HOST = self.host
         Connection.PORT = self.port
-        self.assert_(Connection())
+        self.assertTrue(Connection())
 
     def test_connect(self):
         self.assertRaises(ConnectionFailure, Connection,
                           "somedomainthatdoesntexist.org", connectTimeoutMS=600)
         self.assertRaises(ConnectionFailure, Connection, self.host, 123456789)
 
-        self.assert_(Connection(self.host, self.port))
+        self.assertTrue(Connection(self.host, self.port))
 
     def test_host_w_port(self):
-        self.assert_(Connection("%s:%d" % (self.host, self.port)))
+        self.assertTrue(Connection("%s:%d" % (self.host, self.port)))
         self.assertRaises(ConnectionFailure, Connection,
                           "%s:1234567" % (self.host,), self.port)
 
@@ -108,7 +111,7 @@ class TestConnection(unittest.TestCase):
         self.assertRaises(InvalidName, make_db, connection, "te/t")
         self.assertRaises(InvalidName, make_db, connection, "te st")
 
-        self.assert_(isinstance(connection.test, Database))
+        self.assertTrue(isinstance(connection.test, Database))
         self.assertEqual(connection.test, connection["test"])
         self.assertEqual(connection.test, Database(connection, "test"))
 
@@ -119,8 +122,8 @@ class TestConnection(unittest.TestCase):
         connection.pymongo_test_mike.test.save({"dummy": u"object"})
 
         dbs = connection.database_names()
-        self.assert_("pymongo_test" in dbs)
-        self.assert_("pymongo_test_mike" in dbs)
+        self.assertTrue("pymongo_test" in dbs)
+        self.assertTrue("pymongo_test_mike" in dbs)
 
     def test_drop_database(self):
         connection = Connection(self.host, self.port)
@@ -128,22 +131,25 @@ class TestConnection(unittest.TestCase):
         self.assertRaises(TypeError, connection.drop_database, 5)
         self.assertRaises(TypeError, connection.drop_database, None)
 
-        connection.pymongo_test.test.save({"dummy": u"object"})
-        dbs = connection.database_names()
-        self.assert_("pymongo_test" in dbs)
-        connection.drop_database("pymongo_test")
-        dbs = connection.database_names()
-        self.assert_("pymongo_test" not in dbs)
+        raise SkipTest("This test often fails due to SERVER-2329")
 
         connection.pymongo_test.test.save({"dummy": u"object"})
         dbs = connection.database_names()
-        self.assert_("pymongo_test" in dbs)
+        self.assertTrue("pymongo_test" in dbs)
+        connection.drop_database("pymongo_test")
+        dbs = connection.database_names()
+        self.assertTrue("pymongo_test" not in dbs)
+
+        connection.pymongo_test.test.save({"dummy": u"object"})
+        dbs = connection.database_names()
+        self.assertTrue("pymongo_test" in dbs)
         connection.drop_database(connection.pymongo_test)
         dbs = connection.database_names()
-        self.assert_("pymongo_test" not in dbs)
+        self.assertTrue("pymongo_test" not in dbs)
 
     def test_copy_db(self):
         c = Connection(self.host, self.port)
+        self.assertTrue(c.in_request())
 
         self.assertRaises(TypeError, c.copy_database, 4, "foo")
         self.assertRaises(TypeError, c.copy_database, "foo", 4)
@@ -156,18 +162,26 @@ class TestConnection(unittest.TestCase):
 
         c.pymongo_test.test.insert({"foo": "bar"})
 
-        self.assertFalse("pymongo_test1" in c.database_names())
-        self.assertFalse("pymongo_test2" in c.database_names())
+        # Due to SERVER-2329, databases may not disappear from a master in a
+        # master-slave pair
+        if not server_is_master_with_slave(c):
+            self.assertFalse("pymongo_test1" in c.database_names())
+            self.assertFalse("pymongo_test2" in c.database_names())
 
         c.copy_database("pymongo_test", "pymongo_test1")
+        # copy_database() didn't accidentally end the request
+        self.assertTrue(c.in_request())
 
-        self.assert_("pymongo_test1" in c.database_names())
+        self.assertTrue("pymongo_test1" in c.database_names())
         self.assertEqual("bar", c.pymongo_test1.test.find_one()["foo"])
 
+        c.end_request()
         c.copy_database("pymongo_test", "pymongo_test2",
                         "%s:%d" % (self.host, self.port))
+        # copy_database() didn't accidentally restart the request
+        self.assertFalse(c.in_request())
 
-        self.assert_("pymongo_test2" in c.database_names())
+        self.assertTrue("pymongo_test2" in c.database_names())
         self.assertEqual("bar", c.pymongo_test2.test.find_one()["foo"])
 
         if version.at_least(c, (1, 3, 3, 1)):
@@ -178,16 +192,19 @@ class TestConnection(unittest.TestCase):
             self.assertRaises(OperationFailure, c.copy_database,
                               "pymongo_test", "pymongo_test1",
                               username="foo", password="bar")
-            self.assertFalse("pymongo_test1" in c.database_names())
+            if not server_is_master_with_slave(c):
+                self.assertFalse("pymongo_test1" in c.database_names())
 
             self.assertRaises(OperationFailure, c.copy_database,
                               "pymongo_test", "pymongo_test1",
                               username="mike", password="bar")
-            self.assertFalse("pymongo_test1" in c.database_names())
+
+            if not server_is_master_with_slave(c):
+                self.assertFalse("pymongo_test1" in c.database_names())
 
             c.copy_database("pymongo_test", "pymongo_test1",
                             username="mike", password="password")
-            self.assert_("pymongo_test1" in c.database_names())
+            self.assertTrue("pymongo_test1" in c.database_names())
             self.assertEqual("bar", c.pymongo_test1.test.find_one()["foo"])
 
     def test_iteration(self):
@@ -266,10 +283,10 @@ class TestConnection(unittest.TestCase):
         Connection("mongodb://user:pass@%s:%d/pymongo_test" %
                    (self.host, self.port))
 
-        self.assert_(Connection("mongodb://%s:%d" %
+        self.assertTrue(Connection("mongodb://%s:%d" %
                                 (self.host, self.port),
                                 slave_okay=True).slave_okay)
-        self.assert_(Connection("mongodb://%s:%d/?slaveok=true;w=2" %
+        self.assertTrue(Connection("mongodb://%s:%d/?slaveok=true;w=2" %
                                 (self.host, self.port)).slave_okay)
         c.admin.system.users.remove({})
         c.pymongo_test.system.users.remove({})
@@ -339,33 +356,33 @@ class TestConnection(unittest.TestCase):
         db.test.insert({"x": 1})
 
         self.assertEqual(dict, c.document_class)
-        self.assert_(isinstance(db.test.find_one(), dict))
+        self.assertTrue(isinstance(db.test.find_one(), dict))
         self.assertFalse(isinstance(db.test.find_one(), SON))
 
         c.document_class = SON
 
         self.assertEqual(SON, c.document_class)
-        self.assert_(isinstance(db.test.find_one(), SON))
+        self.assertTrue(isinstance(db.test.find_one(), SON))
         self.assertFalse(isinstance(db.test.find_one(as_class=dict), SON))
 
         c = Connection(self.host, self.port, document_class=SON)
         db = c.pymongo_test
 
         self.assertEqual(SON, c.document_class)
-        self.assert_(isinstance(db.test.find_one(), SON))
+        self.assertTrue(isinstance(db.test.find_one(), SON))
         self.assertFalse(isinstance(db.test.find_one(as_class=dict), SON))
 
         c.document_class = dict
 
         self.assertEqual(dict, c.document_class)
-        self.assert_(isinstance(db.test.find_one(), dict))
+        self.assertTrue(isinstance(db.test.find_one(), dict))
         self.assertFalse(isinstance(db.test.find_one(), SON))
 
     def test_timeouts(self):
         conn = Connection(self.host, self.port, connectTimeoutMS=300)
-        self.assertEquals(0.3, conn._Connection__pool.conn_timeout)
+        self.assertEqual(0.3, conn._Connection__pool.conn_timeout)
         conn = Connection(self.host, self.port, socketTimeoutMS=300)
-        self.assertEquals(0.3, conn._Connection__pool.net_timeout)
+        self.assertEqual(0.3, conn._Connection__pool.net_timeout)
 
     def test_network_timeout(self):
         no_timeout = Connection(self.host, self.port)
@@ -376,28 +393,24 @@ class TestConnection(unittest.TestCase):
         no_timeout.pymongo_test.test.insert({"x": 1}, safe=True)
 
         # A $where clause that takes a second longer than the timeout
-        where_func = """function (doc) {
-  var d = new Date().getTime() + (%f + 1) * 1000;;
-  var x = new Date().getTime();
-  while (x < d) {
-    x = new Date().getTime();
-  }
-  return true;
-}""" % timeout_sec
+        where_func = delay(timeout_sec + 1)
 
         def get_x(db):
-            return db.test.find().where(where_func).next()["x"]
+            doc = db.test.find().where(where_func).next()
+            return doc["x"]
         self.assertEqual(1, get_x(no_timeout.pymongo_test))
         self.assertRaises(ConnectionFailure, get_x, timeout.pymongo_test)
 
         def get_x_timeout(db, t):
-            return db.test.find(
-                        network_timeout=t).where(where_func).next()["x"]
+            doc = db.test.find(network_timeout=t).where(where_func).next()
+            return doc["x"]
         self.assertEqual(1, get_x_timeout(timeout.pymongo_test, None))
         self.assertRaises(ConnectionFailure, get_x_timeout,
                           no_timeout.pymongo_test, 0.1)
 
     def test_tz_aware(self):
+        self.assertRaises(ConfigurationError, Connection, tz_aware='foo')
+
         aware = Connection(self.host, self.port, tz_aware=True)
         naive = Connection(self.host, self.port)
         aware.pymongo_test.drop_collection("test")
@@ -417,7 +430,7 @@ class TestConnection(unittest.TestCase):
         except:
             # Either mongod was started without --ipv6
             # or the OS doesn't support it (or both).
-            raise SkipTest()
+            raise SkipTest("No IPV6")
 
         # Try a few simple things
         connection = Connection("mongodb://[::1]:%d" % (self.port,))
@@ -431,8 +444,8 @@ class TestConnection(unittest.TestCase):
         connection.pymongo_test_bernie.test.save({"dummy": u"object"})
 
         dbs = connection.database_names()
-        self.assert_("pymongo_test" in dbs)
-        self.assert_("pymongo_test_bernie" in dbs)
+        self.assertTrue("pymongo_test" in dbs)
+        self.assertTrue("pymongo_test_bernie" in dbs)
 
     def test_fsync_lock_unlock(self):
         c = get_connection()
@@ -455,95 +468,171 @@ class TestConnection(unittest.TestCase):
     def test_contextlib(self):
         if sys.version_info < (2, 6):
             raise SkipTest()
+
         import contextlib
 
-        conn = get_connection()
+        conn = get_connection(auto_start_request=False)
         conn.pymongo_test.drop_collection("test")
         conn.pymongo_test.test.insert({"foo": "bar"})
-        self.assertNotEqual(None, conn._Connection__pool.sock)
-        self.assertEqual(0, len(conn._Connection__pool.sockets))
+
+        # The socket used for the previous commands has been returned to the
+        # pool
+        self.assertEqual(1, len(conn._Connection__pool.sockets))
 
         # We need exec here because if the Python version is less than 2.6
         # these with-statements won't even compile.
         exec """
 with contextlib.closing(conn):
-    self.assertEquals("bar", conn.pymongo_test.test.find_one()["foo"])
+    self.assertEqual("bar", conn.pymongo_test.test.find_one()["foo"])
+self.assertEqual(0, len(conn._Connection__pool.sockets))
 """
 
-        self.assertEqual(None, conn._Connection__pool.sock)
-        self.assertEqual(0, len(conn._Connection__pool.sockets))
-
         exec """
 with get_connection() as connection:
-    self.assertEquals("bar", connection.pymongo_test.test.find_one()["foo"])
+    self.assertEqual("bar", connection.pymongo_test.test.find_one()["foo"])
+    # Calling conn.close() has reset the pool
+    self.assertEqual(0, len(connection._Connection__pool.sockets))
 """
 
-        self.assertEqual(None, connection._Connection__pool.sock)
-        self.assertEqual(0, len(connection._Connection__pool.sockets))
+    def get_sock(self, pool):
+        sock_info = pool.get_socket((self.host, self.port))
+        return sock_info
+
+    def assertSameSock(self, pool):
+        sock_info0 = self.get_sock(pool)
+        sock_info1 = self.get_sock(pool)
+        self.assertEqual(sock_info0, sock_info1)
+
+    def assertDifferentSock(self, pool):
+        # We have to hold both SocketInfos at the same time, otherwise the
+        # first will send its socket back to the pool as soon as its ref count
+        # goes to zero, in which case the second SocketInfo we get will have
+        # the same socket as the first.
+        sock_info0 = self.get_sock(pool)
+        sock_info1 = self.get_sock(pool)
+        self.assertNotEqual(sock_info0, sock_info1)
+
+    def assertNoRequest(self, pool):
+        self.assertEqual(NO_REQUEST, pool._get_request_state())
+
+    def assertNoSocketYet(self, pool):
+        self.assertEqual(NO_SOCKET_YET, pool._get_request_state())
+
+    def assertRequestSocket(self, pool):
+        self.assertTrue(isinstance(pool._get_request_state(), SocketInfo))
+        
+    def test_with_start_request(self):
+        conn = get_connection(auto_start_request=False)
+        pool = conn._Connection__pool
+
+        # No request started
+        self.assertNoRequest(pool)
+        self.assertDifferentSock(pool)
+
+        # Start a request
+        request_context_mgr = conn.start_request()
+        self.assertTrue(
+            isinstance(request_context_mgr, object)
+        )
+
+        self.assertNoSocketYet(pool)
+        self.assertSameSock(pool)
+        self.assertRequestSocket(pool)
+
+        # End request
+        request_context_mgr.__exit__(None, None, None)
+        self.assertNoRequest(pool)
+        self.assertDifferentSock(pool)
+
+        # Test the 'with' statement
+        if sys.version_info >= (2, 6):
+            # We need exec here because if the Python version is less than 2.6
+            # these with-statements won't even compile.
+            exec """
+with conn.start_request() as request:
+    self.assertEqual(conn, request.connection)
+    self.assertNoSocketYet(pool)
+    self.assertSameSock(pool)
+    self.assertRequestSocket(pool)
+"""
+
+            # Request has ended
+            self.assertNoRequest(pool)
+            self.assertDifferentSock(pool)
+    
+    def test_auto_start_request(self):
+        for bad_horrible_value in (None, 5, 'hi!'):
+            self.assertRaises(
+                (TypeError, ConfigurationError),
+                lambda: get_connection(auto_start_request=bad_horrible_value)
+            )
+
+        # auto_start_request should default to True
+        conn = get_connection()
+        self.assertTrue(conn.auto_start_request)
+        self.assertTrue(conn.in_request())
+        pool = conn._Connection__pool
+
+        # Request started already, just from Connection constructor - it's a
+        # bit weird, but Connection does some socket stuff when it initializes
+        # and it ends up with a request socket
+        self.assertRequestSocket(pool)
+        self.assertSameSock(pool)
+
+        conn.end_request()
+        self.assertNoRequest(pool)
+        self.assertDifferentSock(pool)
+
+        # Trigger auto_start_request
+        conn.db.test.find_one()
+        self.assertRequestSocket(pool)
+        self.assertSameSock(pool)
 
     def test_interrupt_signal(self):
+        if sys.platform.startswith('java'):
+            # We can't figure out how to raise an exception on a thread that's
+            # blocked on a socket, whether that's the main thread or a worker,
+            # without simply killing the whole thread in Jython. This suggests
+            # PYTHON-294 can't actually occur in Jython.
+            raise SkipTest("Can't test interrupts in Jython")
+
         # Test fix for PYTHON-294 -- make sure Connection closes its
         # socket if it gets an interrupt while waiting to recv() from it.
         c = get_connection()
         db = c.pymongo_test
 
         # A $where clause which takes 1.5 sec to execute
-        where = '''function() {
-            var d = new Date((new Date()).getTime() + 1.5 * 1000);
-            while (d > (new Date())) { }; return true;
-        }'''
+        where = delay(1.5)
 
         # Need exactly 1 document so find() will execute its $where clause once
         db.drop_collection('foo')
         db.foo.insert({'_id': 1}, safe=True)
 
-        old_signal_handler = None
+        def interrupter():
+            # Raises KeyboardInterrupt in the main thread
+            time.sleep(0.25)
+            thread.interrupt_main()
 
+        thread.start_new_thread(interrupter, ())
+
+        raised = False
         try:
-            # Platform-specific hacks for raising a KeyboardInterrupt on the main
-            # thread while find() is in-progress: On Windows, SIGALRM is unavailable
-            # so we use second thread. In our Bamboo setup on Linux, the thread
-            # technique causes an error in the test at sock.recv():
-            #    TypeError: 'int' object is not callable
-            # We don't know what causes this in Bamboo, so we hack around it.
-            if sys.platform == 'win32':
-                def interrupter():
-                    time.sleep(0.25)
+            # Will be interrupted by a KeyboardInterrupt.
+            db.foo.find({'$where': where}).next()
+        except KeyboardInterrupt:
+            raised = True
 
-                    # Raises KeyboardInterrupt in the main thread
-                    thread.interrupt_main()
+        # Can't use self.assertRaises() because it doesn't catch system
+        # exceptions
+        self.assertTrue(raised, "Didn't raise expected KeyboardInterrupt")
 
-                thread.start_new_thread(interrupter, ())
-            else:
-                # Convert SIGALRM to SIGINT -- it's hard to schedule a SIGINT for one
-                # second in the future, but easy to schedule SIGALRM.
-                def sigalarm(num, frame):
-                    raise KeyboardInterrupt
-
-                old_signal_handler = signal.signal(signal.SIGALRM, sigalarm)
-                signal.alarm(1)
-
-            raised = False
-            try:
-                # Will be interrupted by a KeyboardInterrupt.
-                db.foo.find({'$where': where}).next()
-            except KeyboardInterrupt:
-                raised = True
-
-            # Can't use self.assertRaises() because it doesn't catch system
-            # exceptions
-            self.assert_(raised, "Didn't raise expected KeyboardInterrupt")
-
-            # Raises AssertionError due to PYTHON-294 -- Mongo's response to the
-            # previous find() is still waiting to be read on the socket, so the
-            # request id's don't match.
-            self.assertEqual(
-                {'_id': 1},
-                db.foo.find().next()
-            )
-        finally:
-            if old_signal_handler:
-                signal.signal(signal.SIGALRM, old_signal_handler)
+        # Raises AssertionError due to PYTHON-294 -- Mongo's response to the
+        # previous find() is still waiting to be read on the socket, so the
+        # request id's don't match.
+        self.assertEqual(
+            {'_id': 1},
+            db.foo.find().next()
+        )
 
 if __name__ == "__main__":
     unittest.main()
