@@ -1,4 +1,4 @@
-# Copyright 2009-2010 10gen, Inc.
+# Copyright 2009-2012 10gen, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -26,9 +26,12 @@ from nose.plugins.skip import SkipTest
 
 from bson.errors import InvalidId
 from bson.objectid import ObjectId
+from bson.py3compat import b, binary_type
 from bson.tz_util import (FixedOffset,
                           utc)
 
+PY3 = sys.version_info[0] == 3
+
 
 def oid(x):
     return ObjectId()
@@ -47,10 +50,10 @@ class TestObjectId(unittest.TestCase):
         self.assertRaises(InvalidId, ObjectId, "")
         self.assertRaises(InvalidId, ObjectId, "12345678901")
         self.assertRaises(InvalidId, ObjectId, "1234567890123")
-        self.assert_(ObjectId())
-        self.assert_(ObjectId("123456789012"))
+        self.assertTrue(ObjectId())
+        self.assertTrue(ObjectId(b("123456789012")))
         a = ObjectId()
-        self.assert_(ObjectId(a))
+        self.assertTrue(ObjectId(a))
 
     def test_unicode(self):
         a = ObjectId()
@@ -69,19 +72,19 @@ class TestObjectId(unittest.TestCase):
                          "ObjectId('1234567890abcdef12345678')")
         self.assertEqual(str(ObjectId("1234567890abcdef12345678")),
                          "1234567890abcdef12345678")
-        self.assertEqual(str(ObjectId("123456789012")),
+        self.assertEqual(str(ObjectId(b("123456789012"))),
                          "313233343536373839303132")
         self.assertEqual(ObjectId("1234567890abcdef12345678").binary,
-                         '\x124Vx\x90\xab\xcd\xef\x124Vx')
-        self.assertEqual(str(ObjectId('\x124Vx\x90\xab\xcd\xef\x124Vx')),
+                         b('\x124Vx\x90\xab\xcd\xef\x124Vx'))
+        self.assertEqual(str(ObjectId(b('\x124Vx\x90\xab\xcd\xef\x124Vx'))),
                          "1234567890abcdef12345678")
 
     def test_cmp(self):
         a = ObjectId()
         self.assertEqual(a, ObjectId(a))
-        self.assertEqual(ObjectId("123456789012"), ObjectId("123456789012"))
+        self.assertEqual(ObjectId(b("123456789012")), ObjectId(b("123456789012")))
         self.assertNotEqual(ObjectId(), ObjectId())
-        self.assertNotEqual(ObjectId("123456789012"), "123456789012")
+        self.assertNotEqual(ObjectId(b("123456789012")), b("123456789012"))
 
     def test_binary_str_equivalence(self):
         a = ObjectId()
@@ -107,7 +110,7 @@ class TestObjectId(unittest.TestCase):
         map = {}
 
         for id in ids:
-            self.assert_(id not in map)
+            self.assertTrue(id not in map)
             map[id] = True
 
     def test_generation_time(self):
@@ -116,9 +119,12 @@ class TestObjectId(unittest.TestCase):
 
         self.assertEqual(utc, d2.tzinfo)
         d2 = d2.replace(tzinfo=None)
-        self.assert_(d2 - d1 < datetime.timedelta(seconds=2))
+        self.assertTrue(d2 - d1 < datetime.timedelta(seconds=2))
 
     def test_from_datetime(self):
+        if 'PyPy 1.8.0' in sys.version:
+            # See https://bugs.pypy.org/issue1092
+            raise SkipTest("datetime.timedelta is broken in pypy 1.8.0")
         d = datetime.datetime.utcnow()
         d = d - datetime.timedelta(microseconds=d.microsecond)
         oid = ObjectId.from_datetime(d)
@@ -133,13 +139,19 @@ class TestObjectId(unittest.TestCase):
 
     def test_pickling(self):
         orig = ObjectId()
-        self.assertEqual(orig, pickle.loads(pickle.dumps(orig)))
+        for protocol in [0, 1, 2, -1]:
+            pkl = pickle.dumps(orig, protocol=protocol)
+            self.assertEqual(orig, pickle.loads(pkl))
 
     def test_pickle_backwards_compatability(self):
+        # For a full discussion see http://bugs.python.org/issue6137
+        if sys.version.startswith('3.0'):
+            raise SkipTest("Python 3.0.x can't unpickle "
+                           "objects pickled in Python 2.x.")
 
         # This string was generated by pickling an ObjectId in pymongo
         # version 1.9
-        pickled_with_1_9 = (
+        pickled_with_1_9 = b(
             "ccopy_reg\n_reconstructor\np0\n"
             "(cbson.objectid\nObjectId\np1\nc__builtin__\n"
             "object\np2\nNtp3\nRp4\n"
@@ -149,15 +161,20 @@ class TestObjectId(unittest.TestCase):
         # We also test against a hardcoded "New" pickle format so that we
         # make sure we're backward compatible with the current version in
         # the future as well.
-        pickled_with_1_10 = (
+        pickled_with_1_10 = b(
             "ccopy_reg\n_reconstructor\np0\n"
             "(cbson.objectid\nObjectId\np1\nc__builtin__\n"
             "object\np2\nNtp3\nRp4\n"
             "S'M\\x9afV\\x13v\\xc0\\x0b\\x88\\x00\\x00\\x00'\np5\nb."
             )
 
-        oid_1_9 = pickle.loads(pickled_with_1_9)
-        oid_1_10 = pickle.loads(pickled_with_1_10)
+        if PY3:
+            # Have to load using 'latin-1' since these were pickled in python2.x.
+            oid_1_9 = pickle.loads(pickled_with_1_9, encoding='latin-1')
+            oid_1_10 = pickle.loads(pickled_with_1_10, encoding='latin-1')
+        else:
+            oid_1_9 = pickle.loads(pickled_with_1_9)
+            oid_1_10 = pickle.loads(pickled_with_1_10)
 
         self.assertEqual(oid_1_9, ObjectId("4d9a66561376c00b88000000"))
         self.assertEqual(oid_1_9, oid_1_10)
