@@ -1,4 +1,4 @@
-# Copyright 2009-2011 10gen, Inc.
+# Copyright 2011-2012 10gen, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -48,9 +48,15 @@ class TestURI(unittest.TestCase):
         self.assertRaises(InvalidURI, parse_userinfo,
                           'fo::o:p@ssword')
         self.assertRaises(InvalidURI, parse_userinfo, ':')
-        self.assert_(parse_userinfo('user:password'))
+        self.assertTrue(parse_userinfo('user:password'))
         self.assertEqual(('us:r', 'p@ssword'),
                          parse_userinfo('us%3Ar:p%40ssword'))
+        self.assertEqual(('us er', 'p ssword'),
+                         parse_userinfo('us+er:p+ssword'))
+        self.assertEqual(('us er', 'p ssword'),
+                         parse_userinfo('us%20er:p%20ssword'))
+        self.assertEqual(('us+er', 'p+ssword'),
+                         parse_userinfo('us%2Ber:p%2Bssword'))
 
     def test_split_hosts(self):
         self.assertRaises(ConfigurationError, split_hosts,
@@ -78,27 +84,26 @@ class TestURI(unittest.TestCase):
         self.assertRaises(ConfigurationError, split_options, 'socketTimeoutMS=0.0')
         self.assertRaises(ConfigurationError, split_options, 'connectTimeoutMS=foo')
         self.assertRaises(ConfigurationError, split_options, 'connectTimeoutMS=0.0')
-        self.assert_(split_options, 'socketTimeoutMS=300')
-        self.assert_(split_options, 'connectTimeoutMS=300')
-        self.assertEquals({'sockettimeoutms': 0.3}, split_options('socketTimeoutMS=300'))
-        self.assertEquals({'sockettimeoutms': 0.0001}, split_options('socketTimeoutMS=0.1'))
-        self.assertEquals({'connecttimeoutms': 0.3}, split_options('connectTimeoutMS=300'))
-        self.assertEquals({'connecttimeoutms': 0.0001}, split_options('connectTimeoutMS=0.1'))
-        self.assert_(split_options, 'connectTimeoutMS=300')
+        self.assertTrue(split_options('socketTimeoutMS=300'))
+        self.assertTrue(split_options('connectTimeoutMS=300'))
+        self.assertEqual({'sockettimeoutms': 0.3}, split_options('socketTimeoutMS=300'))
+        self.assertEqual({'sockettimeoutms': 0.0001}, split_options('socketTimeoutMS=0.1'))
+        self.assertEqual({'connecttimeoutms': 0.3}, split_options('connectTimeoutMS=300'))
+        self.assertEqual({'connecttimeoutms': 0.0001}, split_options('connectTimeoutMS=0.1'))
+        self.assertTrue(split_options('connectTimeoutMS=300'))
         self.assertTrue(isinstance(split_options('w=5')['w'], int))
         self.assertTrue(isinstance(split_options('w=5.5')['w'], basestring))
-        self.assert_(split_options, 'w=foo')
-        self.assert_(split_options, 'w=majority')
+        self.assertTrue(split_options('w=foo'))
+        self.assertTrue(split_options('w=majority'))
         self.assertRaises(ConfigurationError, split_options, 'wtimeoutms=foo')
         self.assertRaises(ConfigurationError, split_options, 'wtimeoutms=5.5')
-        self.assert_(split_options, 'wtimeoutms=500')
+        self.assertTrue(split_options('wtimeoutms=500'))
         self.assertRaises(ConfigurationError, split_options, 'fsync=foo')
         self.assertRaises(ConfigurationError, split_options, 'fsync=5.5')
         self.assertEqual({'fsync': True}, split_options('fsync=true'))
         self.assertEqual({'fsync': False}, split_options('fsync=false'))
-        self.assertRaises(ConfigurationError, split_options, 'maxpoolsize=foo')
-        self.assertRaises(ConfigurationError, split_options, 'maxpoolsize=5.5')
-        self.assert_(split_options, 'maxpoolsize=50')
+        # maxPoolSize isn't yet a documented URI option.
+        self.assertRaises(ConfigurationError, split_options, 'maxpoolsize=50')
 
     def test_parse_uri(self):
         self.assertRaises(InvalidURI, parse_uri, "http://foobar.com")
